using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.XR;
using System.Text;
using MeshVR;


namespace MacGruber
{
	[DefaultExecutionOrder(100)]
	public class CameraLock : MVRScript
	{
		private Transform myTargetTransform;
		private Transform myRigTransform;
		private Transform myCameraTransform;
		private OVRCameraRig myOVRCameraRig;
		
		private JSONStorableString myTargetAtom;
		private bool myIsActive = false;
		
		public override void Init()
		{		
			myTargetAtom = new JSONStorableString("SetTargetAtom", "", SetTargetAtomAction);
			myTargetAtom.isStorable = myTargetAtom.isRestorable = false;
			RegisterString(myTargetAtom);
			
			Utils.SetupAction(this, "ClearTargetAtom", ClearTargetAtomAction);
		}
				
		private void OnDisable()
		{			
			SetTargetTransform(null);
		}
		
		private void SetTargetAtomAction(string atomName)
		{
			myTargetAtom.valNoCallback = string.Empty;
			myTargetTransform = null;
			Atom atom = GetAtomById(atomName);
			if (atom != null)
			{
				SetTargetTransform(atom.mainController.control);
			}
			else
			{
				SuperController.LogError("Can't set CameraLock to unknown atom '"+atomName+"'.");
				SetTargetTransform(null);
			}
		}
		
		private void ClearTargetAtomAction()
		{
			SetTargetTransform(null);
		}
		
		private void SetTargetTransform(Transform targetTransform)
		{	
			myTargetTransform = targetTransform;
			bool isActive = targetTransform != null;		
			if (myIsActive == isActive)
				return;
			
			SuperController sc = SuperController.singleton;
			GlobalSceneOptions.singleton.disableNavigation = isActive;
				
			if (sc.isOVR)
			{		
				if (myRigTransform == null)
				{
					myOVRCameraRig = sc.navigationRig?.Find("HeightOffset/OVRRig")?.GetComponent<OVRCameraRig>();				
					myRigTransform = myOVRCameraRig?.trackingSpace;
					myCameraTransform = myOVRCameraRig?.centerEyeAnchor;
				}
							
				myRigTransform.localRotation = Quaternion.identity;
				myRigTransform.localPosition = Vector3.zero;
				
				if (isActive)
					myOVRCameraRig.UpdatedAnchors += OnUpdateCameraRigOVR;
				else
					myOVRCameraRig.UpdatedAnchors -= OnUpdateCameraRigOVR;
			}
			else if (sc.isOpenVR)
			{
				if (myRigTransform == null)
				{
					myRigTransform = sc.navigationRig.Find("HeightOffset/SteamVRRig");
					myCameraTransform = myRigTransform.Find("Camera (eye)");
				}
				
				myRigTransform.localRotation = Quaternion.identity;
				myRigTransform.localPosition = Vector3.zero;
				
				if (isActive)
					Application.onBeforeRender += OnUpdateCameraRig;
				else
					Application.onBeforeRender -= OnUpdateCameraRig;
			}
			else
			{
				myRigTransform = null;
				myCameraTransform = sc.MonitorCenterCamera.transform;
			}
			
			myIsActive = isActive;	
		}	

		private void OnUpdateCameraRigOVR(OVRCameraRig rig)
		{
			OnUpdateCameraRig();
		}
		
		private void OnUpdateCameraRig()
		{
			Quaternion r = Quaternion.Inverse(myCameraTransform.localRotation);	
			Vector3 p = r * -myCameraTransform.localPosition;
			myRigTransform.localRotation = r;
			myRigTransform.localPosition = p;
		}
			
		private void LateUpdate()
		{
			if (myTargetTransform == null)
				return;
			
			SuperController sc = SuperController.singleton;		
			sc.ResetNavigationRigPositionRotation();
			Transform navRigTransform = sc.navigationRig;
			navRigTransform.rotation = myTargetTransform.rotation;
			Vector3 targetPosition = myTargetTransform.position;
			Vector3 centerCameraPosition = sc.centerCameraTarget.transform.position;
			Vector3 teleportPosition = targetPosition + (navRigTransform.position - centerCameraPosition);
			navRigTransform.position = new Vector3(teleportPosition.x, 0 , teleportPosition.z);
			sc.playerHeightAdjust += (targetPosition.y - centerCameraPosition.y);
		}
	}
}
